
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#define F_CPU 16000000

#include "lcdLib/u8g.h"
#include "music/imperial.h"

#define DELAY_1	1

#define WIDTH  128
#define HEIGHT 64

#define LEFT1	1
#define RIGHT1	2
#define UP1		3
#define DOWN1	4

#define LEFT2	5
#define RIGHT2	6
#define UP2		7
#define DOWN2	8

u8g_t u8g;				// LCD
int start = -1;
int button = -1;
int seed = 0;

#define NUM_NOTE		8
#define DURATA_NOTA		50

float frecventa_nota[8] = {
	261.63, 293.66, 329.63, 349.23, 392.0, 440.0, 493.88, 523.25
};

void buzzer_reda(int nota) {
	float perioada_nota = 100000 / frecventa_nota[nota];
	int i, j;

	for (i = 0; i < DURATA_NOTA; ++i) {
		PORTC |= (1 << PC2);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0; j < perioada_nota / 2; ++j)
			_delay_us(1);

		PORTC &= ~(1 << PC2);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0 ; j < perioada_nota / 2; ++j)
			_delay_us(1);
	}
}

/* Aprinde led-ul PC0 la apasarea butonului PB0. */
void exemplu(void) {
	/* Setez pinul PB0 ca intrare. */
	DDRA &= ~(1 << PA0);

	/* Activez rezistenta de pull-up pentru pinul PB0. */
	PORTA |= (1 << PA0);
	
	/* Setez pinul PB0 ca intrare. */
	DDRA &= ~(1 << PA1);

	/* Activez rezistenta de pull-up pentru pinul PB0. */
	PORTA |= (1 << PA1);

	/* Setez pinul PC0 ca iesire. */
	DDRC |= (1 << PC0);

	/* Sting LED-ul. */
	PORTC &= ~(1 << PC0);
	
	/* Setez pinul PC0 ca iesire. */
	DDRC |= (1 << PC2);

	/* Sting LED-ul. */
	PORTC &= ~(1 << PC2);

	while (1) {
		/* Daca butonul este apasat. */
		if ((PINA & (1 << PA0)) == 0)
			/* Aprind LED-ul. */
			PORTC |= (1 << PC0);
		else
			/* Sting LED-ul. */
			PORTC &= ~(1 << PC0);
		if ((PINA & (1 << PA1)) == 0)
			/* Aprind LED-ul. */
			buzzer_reda(1);
	}
}


void init_buttons(void)
{
	DDRA = 0xffff;
    PORTA = 0x0000;

	/* Initializarea butoane pentru Player 1 */
	/* Setez pinul PA4 ca intrare. */
	DDRA &= ~(1 << PA4);
	/* Activez rezistenta de pull-up pentru pinul PA4. */
	PORTA |= (1 << PA4);
	/* Setez pinul PA5 ca intrare. */
	DDRA &= ~(1 << PA5);
	/* Activez rezistenta de pull-up pentru pinul PA5. */
	PORTA |= (1 << PA5);
	/* Setez pinul PA6 ca intrare. */
	DDRA &= ~(1 << PA6);
	/* Activez rezistenta de pull-up pentru pinul PA6. */
	PORTA |= (1 << PA6);
	/* Setez pinul PC7 ca intrare. */
	DDRA &= ~(1 << PA7);
	/* Activez rezistenta de pull-up pentru pinul PC7. */
	PORTA |= (1 << PA7);
	
	/* Initializarea butoane pentru Player 2 */
	/* Setez pinul PA0 ca intrare. */
	DDRA &= ~(1 << PA0);
	/* Activez rezistenta de pull-up pentru pinul PA0. */
	PORTA |= (1 << PA0);
	/* Setez pinul PA1 ca intrare. */
	DDRA &= ~(1 << PA1);
	/* Activez rezistenta de pull-up pentru pinul PA1. */
	PORTA |= (1 << PA1);
	/* Setez pinul PA2 ca intrare. */
	DDRA &= ~(1 << PA2);
	/* Activez rezistenta de pull-up pentru pinul PA2. */
	PORTA |= (1 << PA2);
	/* Setez pinul PA3 ca intrare. */
	DDRA &= ~(1 << PA3);
	/* Activez rezistenta de pull-up pentru pinul PA3. */
	PORTA |= (1 << PA3);
}

/* Initializam lcd */
void init_lcd(void) 
{
    MCUCR |= (1 << JTD); 
    MCUCR |= (1 << JTD);
    
    // init LCD - data ports PB0 - PB7, control on PA0 - PA4
    u8g_Init8Bit(&u8g, &u8g_dev_st7920_128x64_4x_8bit,
            PN(1, 0), PN(1, 1), PN(1, 2), PN(1, 3), 
            PN(1, 4), PN(1, 5), PN(1, 6), PN(1, 7), 		// DATA PORTS
            PN(2, 3),    									// ENABLE
            U8G_PIN_NONE,
            U8G_PIN_NONE,
            PN(2, 5),    									// RS / DI
            PN(2, 4),    									// RW
            PN(2, 6));   									// RESET
}

void draw_initial_window(void) 
{
    u8g_SetDefaultForegroundColor(&u8g);
    u8g_SetFont(&u8g, u8g_font_helvB10);
    u8g_DrawStr(&u8g, 5, 25, " Portable Game");
	u8g_DrawStr(&u8g, 5, 40, "Apasati un Buton.");
}

void draw_menu(int current) 
{
    u8g_SetDefaultForegroundColor(&u8g);
    u8g_SetFont(&u8g, u8g_font_helvB08);
    u8g_DrawStr(&u8g, 40, 10, "Meniu");
	if (current == 0) {
		u8g_DrawStr(&u8g, 5, 20, "> X si O");
	} else {
		u8g_DrawStr(&u8g, 5, 20, "  X si O");
	}
	if (current == 1) {
		u8g_DrawStr(&u8g, 5, 30, "> Retine culoarea");
	} else {
		u8g_DrawStr(&u8g, 5, 30, "  Retine culoarea");
	}
	if (current == 2) {
		u8g_DrawStr(&u8g, 5, 40, "> Snake");
	} else {
		u8g_DrawStr(&u8g, 5, 40, "  Snake");
	}
	if (current == 3) {
		u8g_DrawStr(&u8g, 5, 50, "> Connect4");	
	} else {
		u8g_DrawStr(&u8g, 5, 50, " Connect4");	
	}
}

int get_button(void)
{
	if ((PINA & (1 << PA0)) == 0) {
		return DOWN1;
	}
	
	if ((PINA & (1 << PA1)) == 0) {
		return LEFT1;
	}
	
	if ((PINA & (1 << PA2)) == 0) {
		return RIGHT1;
	}
	
	if ((PINA & (1 << PA3)) == 0) {
		return UP1;
	}
	
	if ((PINA & (1 << PA4)) == 0) {
		return DOWN2;
	}
	
	if ((PINA & (1 << PA5)) == 0) {
		return UP2;
	}
	
	if ((PINA & (1 << PA6)) == 0) {
		return RIGHT2;
	}
	
	if ((PINA & (1 << PA7)) == 0) {
		return LEFT2;
	}
	
	return 0;
}

void timer1_init()
{
	TCCR1B |=  (1 << WGM12) | (1 << CS11) | (1 << CS10);
	OCR1A = 250;
	TIMSK1 |= (1 << OCIE1A);
	TCNT1 = 0;
}

/*ISR(TIMER1_COMPA_vect)
{
	if (start == -1) {
		button = get_button();
		if (button != -1) {
			//buzzer_reda(button);
			draw_menu();
			start = 0;
		}
	} else if (start == 0) {
		
	}
}*/

void aprinde_rosu(void) {
	PORTD |= (1 << PD0);
}

void stinge_rosu(void) {
	PORTD &= ~(1 << PD0);
}

void aprinde_verde(void) {
	PORTD |= (1 << PD2);
}

void stinge_verde(void) {
	PORTD &= ~(1 << PD2);
}

void aprinde_albastru(void) {
	PORTD |= (1 << PD3);
}

void stinge_albastru(void) {
	PORTD &= ~(1 << PD3);
}

void initializare_led(void) {
	/* Setez pinul PC0 ca iesire. */
	DDRD |= (1 << PD0);
	/* Sting LED-ul. */
	PORTD &= ~(1 << PD0);
	
	/* Setez pinul PC0 ca iesire. */
	DDRD |= (1 << PD2);
	/* Sting LED-ul. */
	PORTD &= ~(1 << PD2);
	
	/* Setez pinul PC0 ca iesire. */
	DDRD |= (1 << PD3);
	/* Sting LED-ul. */
	PORTD &= ~(1 << PD3);
}

void tic_tac_toe(void) {
	int matrice[3][3];
	int i, j, total1, total2;
	int x, y, posX, posY, player, invalid;
	int button;
	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++) {
			matrice[i][j] = 0;
		}
	}
	u8g_SetDefaultForegroundColor(&u8g);
    u8g_SetFont(&u8g, u8g_font_helvB08);
	posX = 0;
	posY = 0;
	player = 1;
	char line[10], column[10], playerInfo[10];
	int final = 0;
	while (1) {
		stinge_albastru();
		stinge_rosu();
		stinge_verde();
		invalid = 0;
		u8g_SetContrast(&u8g, 20);
        u8g_FirstPage(&u8g);
		if (!final) {
			button = get_button();
		} else {
			button = 0;
		}
		if (button == UP1) {
			if (posY > 0) {
				posY -= 1;
			}
		} else if (button == DOWN1) {
			if (posY < 2) {
				posY += 1;
			}
		} else if (button == LEFT1) {
			if (posX > 0) {
				posX -= 1;
			}
		} else if (button == RIGHT1) {
			if (posX < 2) {
				posX += 1;
			}
		} else if (button == UP2) {
			if (matrice[posX][posY] == 0) {
				matrice[posX][posY] = player;
				if (player == 1) {
					player = 2;
				} else {
					player = 1;
				}
				aprinde_verde();
			} else {
				invalid = 1;
				aprinde_rosu();
			}
		}
		_delay_ms(100);
		do {
			x = 5;
			y = 5;
			for (i = 0; i < 3; i++) {
				y = 5;
				for (j = 0; j < 3; j++) {
					u8g_DrawFrame(&u8g, x, y, 15, 15);
					if (matrice[i][j] == 1) {
						u8g_DrawStr(&u8g, x + 3, y + 10, "X");
					} else if (matrice[i][j] == 2) {
						u8g_DrawStr(&u8g, x + 3, y + 10, "O");
					}
					y += 15;
				}
				x += 15;
			}
			u8g_DrawStr(&u8g, (posX * 15) + 5 + 3, (posY * 15) + 5 + 11, "_");
			sprintf(line, "Linia %d", posX + 1);
			sprintf(column, "Coloana %d", posY + 1);
			if (player == 1) {
				sprintf(playerInfo, "Jucator X");
			} else {
				sprintf(playerInfo, "Jucator O");
			}
			u8g_DrawStr(&u8g, 70, 15, line);
			u8g_DrawStr(&u8g, 70, 30, column);
			u8g_DrawStr(&u8g, 70, 45, playerInfo);
			if (invalid) {
				u8g_DrawStr(&u8g, 50, 60, "Celula invalida");
			}
			total1 = 0;
			total2 = 0;
			for (i = 0; i < 3; i++) {
				if (matrice[0][i] == 1) {
					total1 += 1;
				} else if (matrice[0][i] == 2) {
					total2 += 1;
				}
			}
			if (total1 == 3 || total2 == 3) {
				u8g_DrawVLine(&u8g, 12, 5, 45);
			}
			total1 = 0;
			total2 = 0;
			for (i = 0; i < 3; i++) {
				if (matrice[1][i] == 1) {
					total1 += 1;
				} else if (matrice[1][i] == 2) {
					total2 += 1;
				}
			}
			if (total1 == 3 || total2 == 3) {
				u8g_DrawVLine(&u8g, 27, 5, 45);
			}
			_delay_ms(100);
			total1 = 0;
			total2 = 0;
			for (i = 0; i < 3; i++) {
				if (matrice[2][i] == 1) {
					total1 += 1;
				} else if (matrice[2][i] == 2) {
					total2 += 1;
				}
			}
			if (total1 == 3 || total2 == 3) {
				u8g_DrawVLine(&u8g, 42, 5, 45);
			}
			
			total1 = 0;
			total2 = 0;
			for (i = 0; i < 3; i++) {
				if (matrice[i][0] == 1) {
					total1 += 1;
				} else if (matrice[i][0] == 2) {
					total2 += 1;
				}
			}
			if (total1 == 3 || total2 == 3) {
				u8g_DrawHLine(&u8g, 5, 12, 45);
			}
			total1 = 0;
			total2 = 0;
			for (i = 0; i < 3; i++) {
				if (matrice[i][1] == 1) {
					total1 += 1;
				} else if (matrice[i][1] == 2) {
					total2 += 1;
				}
			}
			if (total1 == 3 || total2 == 3) {
				u8g_DrawHLine(&u8g, 5, 27, 45);
			}
			_delay_ms(100);
			total1 = 0;
			total2 = 0;
			for (i = 0; i < 3; i++) {
				if (matrice[i][2] == 1) {
					total1 += 1;
				} else if (matrice[i][2] == 2) {
					total2 += 1;
				}
			}
			if (total1 == 3 || total2 == 3) {
				u8g_DrawHLine(&u8g, 5, 42, 45);
			}
			
			total1 = 0;
			total2 = 0;
			for (i = 0; i < 3; i++) {
				if (matrice[i][i] == 1) {
					total1 += 1;
				} else if (matrice[i][i] == 2) {
					total2 += 1;
				}
			}
			if (total1 == 3 || total2 == 3) {
				u8g_DrawLine(&u8g, 5, 5, 50, 50);
			}
			
			total1 = 0;
			total2 = 0;
			for (i = 0; i < 3; i++) {
				if (matrice[i][2-i] == 1) {
					total1 += 1;
				} else if (matrice[i][2-i] == 2) {
					total2 += 1;
				}
			}
			if (total1 == 3 || total2 == 3) {
				u8g_DrawLine(&u8g, 5, 50, 50, 5);
			}
			
			
			_delay_ms(100);
        } while (u8g_NextPage(&u8g));  
		seed += 1;
		seed %= 255;
	}
}

void connect4(void) {
	int i, j;
	int matrice[4][9];
	int x, y, column, player = 1, button, line, select;
	for (i = 0; i < 4; i++) {
		for (j = 0; j < 9; j++) {
			matrice[i][j] = 0;
		}
	}
	column = 0;
	select = 0;
	while (1) {
		stinge_albastru();
		stinge_rosu();
		stinge_verde();
		u8g_SetContrast(&u8g, 20);
        u8g_FirstPage(&u8g);
		button = get_button();
		if (button == LEFT1) {
			if (column > 0) {
				column--;
			}
		} else if (button == RIGHT1) {
			if (column < 8) {
				column++;
			}
		} else if (button == DOWN1) {
			line = 0;
			if (matrice[0][column] == 0) {
				aprinde_verde();
				select = 1;
			} else {
				aprinde_rosu();
			}
		}
		do {
			x = 10;
			y = 10;
			for (i = 0; i < 4; i++) {
				x = 5;
				for (j = 0; j < 9; j++) {
					u8g_DrawFrame(&u8g, x, y, 10, 10);
					if (matrice[i][j] == 1) {
						u8g_DrawStr(&u8g, x + 2, y + 8, "X");
					} else if (matrice[i][j] == 2) {
						u8g_DrawStr(&u8g, x + 2, y + 8, "O");
					}
					x += 10;
				}
				y += 10;
			}
			if (player == 1) {
				u8g_DrawStr(&u8g, column * 10 + 5, 8, "X");
			} else {
				u8g_DrawStr(&u8g, column * 10 + 5, 8, "O");
			}
			if (select == 1) {
				if (line < 4 && matrice[line][column] == 0) {
					if (player == 1)
						u8g_DrawStr(&u8g, column * 10 + 5, line * 10 + 8, "X");
					else
						u8g_DrawStr(&u8g, column * 10 + 5, line * 10 + 8, "O");
					line += 1;
					_delay_ms(800);
				} else {
					matrice[line-1][column] = player;
					select = 0;
					if (player == 1) {
						player = 2;
					} else {
						player = 1;
					}
					column = 0;
				}
			}
			/*u8g_DrawStr(&u8g, (posX * 15) + 5 + 3, (posY * 15) + 5 + 11, "_");
			sprintf(line, "Linia %d", posX + 1);
			sprintf(column, "Coloana %d", posY + 1);
			if (player == 1) {
				sprintf(playerInfo, "Jucator X");
			} else {
				sprintf(playerInfo, "Jucator O");
			}
			u8g_DrawStr(&u8g, 70, 15, line);
			u8g_DrawStr(&u8g, 70, 30, column);
			u8g_DrawStr(&u8g, 70, 45, playerInfo);
			if (invalid) {
				u8g_DrawStr(&u8g, 50, 60, "Celula invalida");
			}*/
        } while (u8g_NextPage(&u8g));  
		seed += 1;
		seed %= 255;
	}
}

void memory_game(void) {
	int colors[100];
	int size = 0, button, i, lives, score;
	char buffer1[20], buffer2[20];
	lives = 3;
	score = 0;
	u8g_SetDefaultForegroundColor(&u8g);
    u8g_SetFont(&u8g, u8g_font_helvB08);
	while (1) {
		u8g_SetContrast(&u8g, 20);
        u8g_FirstPage(&u8g);
		stinge_albastru();
		stinge_rosu();
		stinge_verde();
		colors[size++] = seed % 3;
		for (i = 0; i < size; i++) {
			if (colors[i] == 0) {
				aprinde_rosu();
			} else if (colors[i] == 1) {
				aprinde_verde();
			} else if (colors[i] == 2) {
				aprinde_albastru();
			} 
			_delay_ms(1000);
			stinge_rosu();
			stinge_verde();
			stinge_albastru();
			seed += 1;
			seed %= 255;
		}
        do {
			u8g_DrawStr(&u8g, 10, 10, "< Rosu");
			u8g_DrawStr(&u8g, 10, 20, "^ Verde");
			u8g_DrawStr(&u8g, 10, 30, "> Albastru");
			if (lives == 3) {
				sprintf(buffer1, "Vieti: ooo");
			} else if (lives == 2) {
				sprintf(buffer1, "Vieti: oo");
			} else if (lives == 0) {
				sprintf(buffer1, "Vieti: o");
			}
			u8g_DrawStr(&u8g, 70, 10, buffer1);
			sprintf(buffer2, "Scor: %d", score);
			u8g_DrawStr(&u8g, 70, 20, buffer2);
			seed += 1;
			seed %= 255;
            //draw_initial_window();
        } while (u8g_NextPage(&u8g));
		i = 0;
		while (i < size) {
			stinge_rosu();
			stinge_verde();
			stinge_albastru();
			button = get_button();
			if (button == LEFT1 && colors[i] != 0) {
				lives -= 1;
				break;
			} else if (button == UP1 && colors[i] != 1) {
				lives -= 1;
				break;
			} else if (button == RIGHT1 && colors[i] != 2) {
				lives -= 1;
				break;
			} else if (button == LEFT1 && colors[i] == 0) {
				i += 1;
				aprinde_rosu();
			} else if (button == RIGHT1 && colors[i] == 2) {
				i += 1;
				aprinde_albastru();
			} else if (button == UP1 && colors[i] == 1) {
				i += 1;
				aprinde_verde();
			}
			seed += 1;
			seed %= 255;
			_delay_ms(1000);
		}
		if (i == size) {
			score += size;
		}
		seed += 1;
		seed %= 255;
	}
}

int main(void) 
{
	init_lcd();
	init_buttons();
	initializare_led();
	//timer1_init();
	int button = -1;
	/* Setez pinul PD0 ca iesire. */
	DDRD |= (1 << PD1);
	/* Opresc buzzer-ul. */
	PORTD &= ~(1 << PD1);
	int current = 0;
	
	play_imperial();
	
	while(1) {
		u8g_SetContrast(&u8g, 20);
        u8g_FirstPage(&u8g);
        do {
            draw_initial_window();
        } while (u8g_NextPage(&u8g));  
		button = get_button();
		if (button > 0) {
			break;
		}
		seed += 1;
		seed %= 255;
		_delay_ms(100);
    }
	
	while(1) {
		u8g_SetContrast(&u8g, 20);
        u8g_FirstPage(&u8g);
        do {
            draw_menu(current);
        } while (u8g_NextPage(&u8g)); 
		button = get_button();
		if (button == UP1) {
			if (current == 0) {
				current = 3; 
			} else {
				current -= 1;
			}
		}
		if (button == DOWN1) {
			current += 1;
			current = current % 4;
		}
		if (button == RIGHT1) {
			break;
		}
		seed += 1;
		seed %= 255;
		_delay_ms(100);
    }
	
	/*while (1) {
		button = get_button();
		stinge_rosu();
		stinge_verde();
		stinge_albastru();
		if (button == UP1) {
			aprinde_rosu();
		} else if (button == DOWN1) {
			aprinde_albastru();
		} else if (button == RIGHT1) {
			aprinde_verde();
		}
		
		if (button == LEFT2) {
			aprinde_verde();
		} else if (button == UP2) {
			aprinde_albastru();
		} else if (button == RIGHT2) {
			aprinde_rosu();
		} else if (button == DOWN2) {
			aprinde_verde();
		}
		_delay_us(1000);
	}*/
	
	if (current == 3) {
		connect4();
	} else if (current == 0) {
		tic_tac_toe();
	} else if (current == 1) {
		memory_game();
	}
	//exemplu();
	
	/*while(1) {
		button = get_button();
		if (button >= 0) {
			buzzer_reda(button);
		}
		PORTD &= ~(1 << PD0);
		_delay_us(2000);
	}*/
	
    return 0;
}